@@ -16,6 +16,11 @@ class ZIPTableSet(messytables.TableSet):
         try:
             for f in z.infolist():
                 ext = None
+
+                #ignore metadata folders added by Mac OS X
+                if '__MACOSX' in f.filename:
+                    continue
+
                 if "." in f.filename:
                     ext = f.filename[f.filename.rindex(".") + 1:]
 
@@ -33,11 +38,5 @@ class ZIPTableSet(messytables.TableSet):
                     tables (%s).''' % ', '.join(found))
         finally:
             z.close()
+        
         self._tables = tables
-
-    @property
-    def tables(self):
-        """ Return the tables contained in any loadable
-        files within the ZIP file.
-        """
-        return self._tables
