@@ -2,20 +2,23 @@
 import unittest
 
 from . import horror_fobj
-from nose.tools import assert_equal, assert_true
+from nose.plugins.attrib import attr
+from nose.tools import assert_equal
+from nose.plugins.skip import SkipTest
+
 try:
-    # Python 2.6 doesn't provide assert_is_instance
-    from nose.tools import assert_is_instance
+    # Python 2.6 doesn't provide these functions
+    from nose.tools import assert_is_instance, assert_greater_equal
 except ImportError:
-    def assert_is_instance(obj, cls, msg=None):
-        assert_true(isinstance(obj, cls))
+    from shim26 import assert_is_instance, assert_greater_equal
 
 from messytables import (CSVTableSet, StringType, HTMLTableSet,
                          ZIPTableSet, XLSTableSet, XLSXTableSet, PDFTableSet,
                          ODSTableSet, headers_guess, headers_processor,
                          offset_processor, DateType, FloatType,
-                         IntegerType, rowset_as_jts,
-                         types_processor, type_guess)
+                         IntegerType, BoolType, rowset_as_jts,
+                         types_processor, type_guess, ReadError,
+                         null_processor)
 import datetime
 
 
@@ -84,6 +87,7 @@ class ReadCsvTest(unittest.TestCase):
         data = list(row_set)
         assert_equal(int(data[0][1].value), 1)
 
+    @attr("slow")
     def test_read_type_guess_simple(self):
         fh = horror_fobj('simple.csv')
         table_set = CSVTableSet(fh)
@@ -104,7 +108,7 @@ class ReadCsvTest(unittest.TestCase):
         table_set = CSVTableSet(fh)
         row_set = table_set.tables[0]
         types = type_guess(row_set.sample, strict=True)
-        expected_types = [IntegerType(), StringType(), IntegerType(),
+        expected_types = [IntegerType(), StringType(), BoolType(),
                           StringType()]
         assert_equal(types, expected_types)
 
@@ -121,6 +125,32 @@ class ReadCsvTest(unittest.TestCase):
         # we expect None for Integers and "" for empty strings in CSV
         assert [x.value for x in data[2]] == [3, "null", None, ""], data[2]
 
+    def test_null_process(self):
+        fh = horror_fobj('null.csv')
+        table_set = CSVTableSet(fh)
+        row_set = table_set.tables[0]
+        row_set.register_processor(null_processor(['null']))
+        data = list(row_set)
+
+        nones = [[x.value is None for x in row] for row in data]
+        assert_equal(nones[0], [False, True, False, False])
+        assert_equal(nones[1], [False, False, False, True])
+        assert_equal(nones[2], [False, True, False, False])
+
+        types = type_guess(row_set.sample, strict=True)
+        expected_types = [IntegerType(), BoolType(), BoolType(),
+                          BoolType()]
+        assert_equal(types, expected_types)
+
+        row_set.register_processor(types_processor(types))
+
+        # after applying the types, '' should become None for int columns
+        data = list(row_set)
+        nones = [[x.value is None for x in row] for row in data]
+        assert_equal(nones[0], [False, True, False, False])
+        assert_equal(nones[1], [False, False, False, True])
+        assert_equal(nones[2], [False, True, True, True])
+
     def test_read_encoded_csv(self):
         fh = horror_fobj('utf-16le_encoded.csv')
         table_set = CSVTableSet(fh)
@@ -174,6 +204,18 @@ class ReadCsvTest(unittest.TestCase):
         assert 'foo' in data[12][0].column, data[12][0]
         assert 'Chirurgie' in data[12][0].value, data[12][0].value
 
+    def test_read_encoded_characters_csv(self):
+        fh = horror_fobj('characters.csv')
+        table_set = CSVTableSet(fh)
+        row_set = table_set.tables[0]
+        offset, headers = headers_guess(row_set.sample)
+        row_set.register_processor(headers_processor(headers))
+        row_set.register_processor(offset_processor(offset + 1))
+        data = list(row_set)
+        assert_equal(382, len(data))
+        assert_equal(data[0][2].value, u'雲嘉南濱海國家風景區管理處')
+        assert_equal(data[-1][2].value, u'沈光文紀念廳')
+
 
 class ReadZipTest(unittest.TestCase):
     def test_read_simple_zip(self):
@@ -236,6 +278,7 @@ class ReadODSTest(unittest.TestCase):
             assert 3 == len(row), row
         assert_equal(total, 0)
 
+    @attr("slow")
     def test_read_large_ods(self):
         fh = horror_fobj('large.ods')
         table_set = ODSTableSet(fh)
@@ -301,6 +344,18 @@ class ReadXlsTest(unittest.TestCase):
         for row in list(row_set):
             assert 3 == len(row), row
 
+    # Right now we can't read even passwordless encrypted files - in future
+    # would be good to be able to.
+    def test_attempt_read_encrypted_no_password_xls(self):
+        fh = horror_fobj('encrypted_no_password.xls')
+        errmsg = "Can't read Excel file: XLRDError('Workbook is encrypted',)"
+        try:
+            XLSTableSet(fh)
+        except ReadError as e:
+            assert e.message == errmsg
+        else:
+            assert False, "Did not raise Read Error"
+
     def test_read_head_offset_excel(self):
         fh = horror_fobj('simple.xls')
         table_set = XLSTableSet(fh)
@@ -398,6 +453,14 @@ class ReadHtmlTest(unittest.TestCase):
         assert_equal(row[1].value.strip(), 'Country')
         assert_equal(row[4].value.strip(), '2010')
 
+    def test_invisible_text_html(self):
+        fh = horror_fobj('invisible_text.html')
+        table_set = HTMLTableSet(fh)
+        row_set = table_set.tables[0]
+        assert_equal(4, len(list(row_set)))
+        row = list(row_set.sample)[1]
+        assert_equal(row[5].value.strip(), '1 July 1879')
+
     def test_read_span_html(self):
         fh = horror_fobj('rowcolspan.html')
         table_set = HTMLTableSet(fh)
@@ -471,13 +534,20 @@ class ReadPdfTest(unittest.TestCase):
                 PDFTableSet(fh)
             except ImportError:
                 # Optional library isn't installed. Skip the tests.
-                self.skipTest("pdftables is not installed, skipping PDF tests")
+                raise SkipTest("pdftables is not installed, skipping PDF tests")
+
 
     def test_read_simple_pdf(self):
         with horror_fobj('simple.pdf') as fh:
             table_set = PDFTableSet(fh)
+
         assert_equal(1, len(list(table_set.tables)))
 
+        (table,) = table_set.tables
+        rows = list(table)
+
+        assert_greater_equal(len(rows), 1)
+
     def test_pdf_names(self):
         with horror_fobj('simple.pdf') as fh:
             table_set = PDFTableSet(fh)
